Package: Cascade
Type: Package
Title: Selection, Reverse-Engineering and Prediction in Cascade
        Networks
Version: 2.4
Date: 2026-04-30
Depends: R (>= 3.5.0)
biocViews:
Imports: abind, animation, cluster, igraph, lars, lattice, limma,
        methods, nnls, splines, stats4, tnet, VGAM
Suggests: R.rsp, CascadeData, knitr, testthat (>= 3.0.0)
Authors@R: c(
	person(given = "Frederic", family= "Bertrand", role = c("cre", "aut"), email = "frederic.bertrand@lecnam.net", comment = c(ORCID = "0000-0002-0837-8281")),
	person(given = "Myriam", family= "Maumy-Bertrand", role = c("aut"), email = "myriam.maumy@ehesp.fr", comment = c(ORCID = "0000-0002-4615-1512")),
	person(given = "Laurent", family= "Vallat", role = c("ctb"), email = "vallat@unistra.fr"),
	person(given = "Nicolas", family= "Jung", role = c("ctb"), email = "nicolas.jung@unistra.fr"))
Author: Frederic Bertrand [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-0837-8281>),
  Myriam Maumy-Bertrand [aut] (ORCID:
    <https://orcid.org/0000-0002-4615-1512>),
  Laurent Vallat [ctb],
  Nicolas Jung [ctb]
Maintainer: Frederic Bertrand <frederic.bertrand@lecnam.net>
Description: A modeling tool allowing gene selection, reverse engineering, and prediction in cascade networks. Jung, N., Bertrand, F., Bahram, S., Vallat, L., and Maumy-Bertrand, M. (2014) <doi:10.1093/bioinformatics/btt705>.
License: GPL (>= 2)
Encoding: UTF-8
Collate: Cascade-package.R global.R micro_array.R network.R
        micro_array-network.R micropredict.R datasets.R
Classification/MSC: 62J05, 62J07, 62J99, 92C42
VignetteBuilder: R.rsp
RoxygenNote: 7.3.3
URL: https://fbertran.github.io/Cascade/,
        https://github.com/fbertran/Cascade
BugReports: https://github.com/fbertran/Cascade/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-30 08:59:30 UTC; bertran7
Repository: CRAN
Date/Publication: 2026-04-30 10:00:08 UTC
