# Generated by roxygen2: do not edit by hand

export(as.micro_array)
export(network_random)
exportClasses(micro_array)
exportClasses(micropredict)
exportClasses(network)
exportMethods(analyze_network)
exportMethods(compare)
exportMethods(cutoff)
exportMethods(dim)
exportMethods(evolution)
exportMethods(geneNeighborhood)
exportMethods(genePeakSelection)
exportMethods(geneSelection)
exportMethods(gene_expr_simulation)
exportMethods(head)
exportMethods(inference)
exportMethods(plot)
exportMethods(position)
exportMethods(predict)
exportMethods(print)
exportMethods(summary)
exportMethods(unionMicro)
import(grid)
import(igraph)
import(lattice)
import(limma)
import(magic)
import(nnls)
import(splines)
import(stats4)
import(survival)
import(tnet)
importFrom(VGAM,rlaplace)
importFrom(VGAM,rpareto)
importFrom(VGAM,zeta)
importFrom(abind,abind)
importFrom(animation,ani.options)
importFrom(animation,saveHTML)
importFrom(cluster,agnes)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,dev.new)
importFrom(grDevices,grey)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(methods,new)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,model.matrix)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reshape)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,sessionInfo)
