$Id: README,v 1.2 2001/09/05 17:52:46 dan Exp $

Cascade

A program for performing noise figure, gain, and third order intercept
analysis of a cascade.

See doc/index.html for documentation on using the program

See INSTALL for installation help

This program is Copyright 1997-2001 Dan McMahill, please refer
to the file 'COPYING' for details.


