#' Simulated M data for examples.
#' 
#' Simulated M microarray.
#' 
#' 
#' @name M
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(M)
#' head(M)
#' 
NULL

#' Reverse-engineered network of the simulated data.
#' 
#' The reverse-engineered network of the simulated data (M and Net).
#' 
#' 
#' @name Net_inf
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(Net_inf)
#' str(Net_inf)
#' 
NULL

#' Simulated network data for examples.
#' 
#' Simulated network.
#' 
#' 
#' @name Net
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(Net)
#' str(Net)
#' 
NULL

#' A network object data.
#' 
#' A network object. It is the same as the result in the vignette for the
#' inference of the network.
#' 
#' 
#' @name network
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(network)
#' plot(network)
#' print(network)
#' 
NULL

#' Selection of genes.
#' 
#' 20 (at most) genes with differential expression at t1, 20 (at most) genes
#' with differential expression at t2, 20 (at most) genes with differential
#' expression at t3, 20 (at most) genes with differential expression at t4 et
#' 20 (at most) genes with global differential expression were selected.
#' 
#' 
#' @name Selection
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(Selection)
#' head(Selection)
#' summary(Selection,3)
#' 
NULL
