% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Net}
\alias{Net}
\title{Simulated network data for examples.}
\description{
Simulated network.
}
\examples{

data(Net)
str(Net)

}
\keyword{datasets}
